@@ -1,6 +1,6 @@
 ;; data.scm --- Data structures for gzochi AberMUD example game
 
-;; Copyright (C) 2012 Julian Graham
+;; Copyright (C) 2013 Julian Graham
 ;;
 ;; This software is provided 'as-is', without any express or implied
 ;; warranty. In no event will the authors be held liable for any damages
@@ -317,12 +317,10 @@
 	   (mutable next abermud:item-next abermudmud:set-item-next!)
 	   (immutable facets))
 
-   (nongenerative abermud:item)
-   (protocol (lambda (n)
+   (protocol (lambda (p)
 	       (lambda (adjective noun)
-		 (let ((p (n)))
-		   (p adjective noun #f #f #f
-		      (gzochi:make-managed-vector 10)))))))
+		 (p adjective noun #f #f #f
+		    (gzochi:make-managed-vector 10))))))
 
   ;; Add `child' to the set of children of `parent' and make `parent' the
   ;; parent item of `child'.
@@ -411,8 +409,7 @@
    (abermud:inouthere abermud:make-inouthere abermud:inouthere?)
    (fields (immutable in (serialization gzochi:string-serialization))
 	   (immutable out (serialization gzochi:string-serialization))
-	   (immutable here (serialization gzochi:string-serialization)))
-   (nongenerative abermud:inouthere))	   
+	   (immutable here (serialization gzochi:string-serialization))))
 
   ;; The object facet. Represents a non-player object in the gzochi abermud
   ;; game world. Includes a text description and a set of flags that describe
@@ -424,8 +421,7 @@
    (fields (immutable text (serialization gzochi:string-serialization))
 	   (immutable flags (serialization 
 			     (gzochi:make-uniform-list-serialization 
-			      gzochi:symbol-serialization))))
-   (nongenerative abermud:object))
+			      gzochi:symbol-serialization)))))
 
   ;; The player facet. Represents a player in the gzochi abermud game world,
   ;; independent of whether an active gzochi client session is "attached" to
@@ -440,8 +436,7 @@
 		    abermud:set-player-password!
 		    (serialization gzochi:string-serialization)))
 
-   (nongenerative abermud:player)
-   (protocol (lambda (n) (lambda (name) (let ((p (n))) (p name ""))))))
+   (protocol (lambda (p) (lambda (name) (p name "")))))
 
   ;; The room facet. Rooms act as containers (via their parent-child 
   ;; relationships) with other items in the game world. A room has a short text
@@ -460,14 +455,12 @@
 		      (serialization gzochi:string-serialization))
 	   channel
 	   exits)
-   (nongenerative abermud:room)
-   (protocol (lambda (n)
+   (protocol (lambda (p)
 	       (lambda (short-description long-description)
-		 (let ((p (n)))
-		   (p short-description 
-		      long-description 
-		      (gzochi:create-channel (symbol->string (gensym)))
-		      (gzochi:make-managed-vector 12)))))))
+		 (p short-description 
+		    long-description 
+		    (gzochi:create-channel (symbol->string (gensym)))
+		    (gzochi:make-managed-vector 12))))))
   
   ;; Convert the symbolic representation of an exit direction to the index of
   ;; a directional exit in a room facet's exit vector.
@@ -498,10 +491,9 @@
    (fields (immutable destination)
 	   (immutable message (serialization gzochi:string-serialization)))
 
-   (nongenerative abermud:exit)
-   (protocol (lambda (n)
+   (protocol (lambda (p)
 	       (lambda* (destination #:optional message)
-		 (let ((p (n))) (p destination (or message "")))))))
+		 (p destination (or message ""))))))
 
   ;; Returns the exit record for the specified room going in the specified
   ;; direction, or `#f' if no such exit exists.
