@@ -1,5 +1,5 @@
 /* data.c: Primitive functions for user-facing gzochi data management API
- * Copyright (C) 2012 Julian Graham
+ * Copyright (C) 2013 Julian Graham
  *
  * gzochi is free software: you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by
@@ -41,7 +41,7 @@ SCM_DEFINE (primitive_create_reference, "primitive-create-reference", 1, 0, 0,
 
   ret = gzochid_scheme_create_managed_reference (reference);
   
-  gzochid_api_check_rollback ();
+  gzochid_api_check_transaction ();
 
   return ret;
 }
@@ -74,7 +74,7 @@ SCM_DEFINE (primitive_dereference, "primitive-dereference", 1, 0, 0, (SCM ref),
       ret = gzochid_scm_location_resolve (context, location);
     }
 
-  gzochid_api_check_rollback ();
+  gzochid_api_check_transaction ();
 
   return ret;
 }
@@ -121,7 +121,7 @@ SCM_DEFINE (primitive_get_binding, "primitive-get-binding", 1, 0, 0, (SCM name),
       free (prefixed_name);
     }
 
-  gzochid_api_check_rollback ();
+  gzochid_api_check_transaction ();
   return ret;
 }
 
@@ -147,6 +147,8 @@ SCM_DEFINE (primitive_set_binding_x, "primitive-set-binding!", 2, 0, 0,
       gzochid_scm_location_info *obj_loc = 
 	gzochid_scm_location_get (context, obj);
 
+      scm_gc_protect_object (obj);
+      
       gzochid_data_set_binding 
 	(context, prefixed_name, &gzochid_scm_location_aware_serialization, 
 	 obj_loc);
@@ -155,7 +157,7 @@ SCM_DEFINE (primitive_set_binding_x, "primitive-set-binding!", 2, 0, 0,
       free (prefixed_name);
     }
 
-  gzochid_api_check_rollback ();
+  gzochid_api_check_transaction ();
   return SCM_UNSPECIFIED;
 }
 
@@ -184,7 +186,7 @@ SCM_DEFINE (primitive_remove_binding_x, "primitive-remove-binding!", 1, 0, 0,
       free (prefixed_name);
     }
 
-  gzochid_api_check_rollback ();
+  gzochid_api_check_transaction ();
   return SCM_UNSPECIFIED;
 }
 
@@ -197,8 +199,10 @@ SCM_DEFINE (primitive_remove_object_x, "primitive-remove-object!", 1, 0, 0,
   gzochid_data_managed_reference *reference = gzochid_data_create_reference 
     (context, &gzochid_scm_location_aware_serialization, obj_loc);
 
+  scm_gc_protect_object (obj);    
+
   gzochid_data_remove_object (reference);
-  gzochid_api_check_rollback ();
+  gzochid_api_check_transaction ();
 
   return SCM_UNSPECIFIED;
 }
@@ -210,9 +214,11 @@ SCM_DEFINE (primitive_mark_for_write_x, "primitive-mark-for-write!", 1, 0, 0,
     gzochid_api_ensure_current_application_context ();
   gzochid_scm_location_info *obj_loc = gzochid_scm_location_get (context, obj);
 
+  scm_gc_protect_object (obj);    
+
   gzochid_data_mark 
     (context, &gzochid_scm_location_aware_serialization, obj_loc);
-  gzochid_api_check_rollback ();
+  gzochid_api_check_transaction ();
 
   return SCM_UNSPECIFIED;
 }
