@@ -21,7 +21,9 @@
 
 #include "app.h"
 #include "auth.h"
+#include "auth_int.h"
 #include "context.h"
+#include "event.h"
 #include "fsm.h"
 #include "game.h"
 #include "guile.h"
@@ -42,12 +44,67 @@ static GStaticPrivate thread_identity_key = G_STATIC_PRIVATE_INIT;
 
 G_LOCK_DEFINE_STATIC (load_path);
 
+typedef struct _gzochid_event_transaction_context
+{
+  gzochid_application_context *app_context;
+  struct timeval start_time;
+} gzochid_event_transaction_context;
+
 static void initialize_async (gpointer data, gpointer user_data)
 {
   gzochid_context *context = (gzochid_context *) data;
   gzochid_fsm_to_state (context->fsm, GZOCHID_APPLICATION_STATE_RUNNING);
 }
 
+static void initialize_auth (int from_state, int to_state, gpointer user_data)
+{
+  gzochid_context *context = (gzochid_context *) user_data;
+  gzochid_application_context *app_context = 
+    (gzochid_application_context *) context;
+  gzochid_game_context *game_context = (gzochid_game_context *) context->parent;
+
+  if (app_context->descriptor->auth_type != NULL)
+    {
+      if (g_hash_table_contains 
+	  (game_context->auth_plugins, app_context->descriptor->auth_type))
+	{
+	  GError *error = NULL;
+	  gzochid_auth_plugin *plugin = g_hash_table_lookup 
+	    (game_context->auth_plugins, app_context->descriptor->auth_type);
+
+	  gzochid_debug 
+	    ("Initializing auth plugin '%s' for application '%'.",
+	     app_context->descriptor->auth_type,
+	     app_context->descriptor->name);
+
+	  app_context->auth_data = plugin->info->initialize 
+	    (app_context->descriptor->auth_properties, &error);
+	  if (error != NULL)
+	    gzochid_err
+	      ("Auth plugin '%s' failed to initialize "
+	       "for application '%s': %s",
+	       app_context->descriptor->auth_type,
+	       app_context->descriptor->name,
+	       error->message);
+	  
+	  g_clear_error (&error);
+	  app_context->authenticator = plugin->info->authenticate;
+	}
+      else gzochid_err 
+	     ("Unknown auth type '%s' for application '%s'.", 
+	      app_context->descriptor->auth_type,
+	      app_context->descriptor->name);
+    }
+  else 
+    {
+      gzochid_info 
+	("No auth plugin specified for application '%s'; " 
+	 "pass-thru authentication will be used.", 
+	 app_context->descriptor->name);
+      app_context->authenticator = gzochid_auth_function_pass_thru;
+    }
+}
+
 static void initialize_data (int from_state, int to_state, gpointer user_data)
 {
   gzochid_context *context = (gzochid_context *) user_data;
@@ -143,6 +200,121 @@ void gzochid_serialize_application_task
   serialization->data_serialization->serializer (context, task->data, out);
 }
 
+static gzochid_event_transaction_context *create_transaction_context 
+(gzochid_application_context *app_context)
+{
+  gzochid_event_transaction_context *tx_context = 
+    malloc (sizeof (gzochid_event_transaction_context));
+
+  tx_context->app_context = app_context;
+  gettimeofday (&tx_context->start_time, NULL);
+
+  return tx_context;
+}
+
+static int event_prepare (gpointer data)
+{
+  return TRUE;
+}
+
+static void event_commit (gpointer data)
+{
+  gzochid_event_transaction_context *tx_context = 
+    (gzochid_event_transaction_context *) data;
+  gzochid_application_transaction_event *event = 
+    malloc (sizeof (gzochid_application_transaction_event));
+  gzochid_application_event *base_event = (gzochid_application_event *) event;
+  struct timeval now;
+
+  gettimeofday (&now, NULL);
+  base_event->type = TRANSACTION_COMMIT;
+  gettimeofday (&base_event->timestamp, NULL);
+  timersub (&now, &tx_context->start_time, &event->duration);
+
+  gzochid_application_event_dispatch 
+    (tx_context->app_context->event_source, base_event);
+  free (tx_context);
+}
+
+static void event_rollback (gpointer data)
+{
+  gzochid_event_transaction_context *tx_context = 
+    (gzochid_event_transaction_context *) data;
+
+  gzochid_application_transaction_event *event = 
+    malloc (sizeof (gzochid_application_transaction_event));
+  gzochid_application_event *base_event = (gzochid_application_event *) event;
+  struct timeval now;
+
+  gettimeofday (&now, NULL);
+  base_event->type = TRANSACTION_ROLLBACK;
+  gettimeofday (&base_event->timestamp, NULL);
+  timersub (&now, &tx_context->start_time, &event->duration);
+
+  gzochid_application_event_dispatch 
+    (tx_context->app_context->event_source, base_event);
+  free (tx_context);
+}
+
+static gzochid_transaction_participant event_participant = 
+  { "event", event_prepare, event_commit, event_rollback };
+
+static void join_transaction (gzochid_application_context *context)
+{
+  if (!gzochid_transaction_active ()
+      || gzochid_transaction_context (&event_participant) == NULL)
+    {
+      gzochid_event_transaction_context *tx_context =
+	create_transaction_context (context); 
+      gzochid_transaction_join (&event_participant, tx_context);
+    }
+}
+
+static void event_func_wrapper (gpointer data)
+{
+  gpointer *args = (gpointer *) data;
+  gzochid_application_context *context = 
+    (gzochid_application_context *) args[0];
+  void (*func) (gpointer) = (void (*) (gpointer)) args[1];
+  gpointer func_data = args[2];
+
+  gzochid_application_event *event = 
+    malloc (sizeof (gzochid_application_event));
+
+  event->type = TRANSACTION_START;
+  gettimeofday (&event->timestamp, NULL);
+
+  join_transaction (context);
+
+  gzochid_application_event_dispatch (context->event_source, event);
+  func (func_data);  
+}
+
+gzochid_transaction_result gzochid_application_transaction_execute 
+(gzochid_application_context *context, void (*func) (gpointer), gpointer data)
+{
+  gpointer args[3];
+
+  args[0] = context;
+  args[1] = func;
+  args[2] = data;
+
+  return gzochid_transaction_execute (event_func_wrapper, args);
+}
+
+gzochid_transaction_result gzochid_application_transaction_execute_timed 
+(gzochid_application_context *context, void (*func) (gpointer), gpointer data,
+ struct timeval timeout)
+{
+  gpointer args[3];
+
+  args[0] = context;
+  args[1] = func;
+  args[2] = data;
+
+  return gzochid_transaction_execute_timed (event_func_wrapper, args, timeout);
+}
+
 static gzochid_transactional_application_task_execution *execution_new
 (gzochid_application_task *task, struct timeval *timeout)
 {
@@ -212,10 +384,10 @@ void gzochid_application_transactional_task_worker
   args[2] = execution->task;
 
   if (execution->timeout != NULL)
-    execution->result = gzochid_transaction_execute_timed 
-      (transactional_task_worker, args, *execution->timeout);
-  else execution->result = 
-	 gzochid_transaction_execute (transactional_task_worker, args);
+    execution->result = gzochid_application_transaction_execute_timed 
+      (context, transactional_task_worker, args, *execution->timeout);
+  else execution->result = gzochid_application_transaction_execute 
+	 (context, transactional_task_worker, args);
 
   execution->attempts++;
 }
@@ -310,7 +482,8 @@ static void run_async_transactional (gpointer data)
 
 static void run_async (gpointer data, gpointer user_data)
 {
-  gzochid_transaction_execute (run_async_transactional, data);
+  gzochid_application_transaction_execute 
+    ((gzochid_application_context *) data, run_async_transactional, data);
 }
 
 static void run (int from_state, int to_state, gpointer user_data)
@@ -416,6 +589,8 @@ gzochid_application_context *gzochid_application_context_new (void)
   context->free_oids_lock = g_mutex_new ();
   context->client_mapping_lock = g_mutex_new ();
 
+  context->event_source = gzochid_application_event_source_new ();
+  context->stats = calloc (1, sizeof (gzochid_application_stats));
   return context;
 }
 
@@ -427,10 +602,18 @@ void gzochid_application_context_free (gzochid_application_context *app_context)
   g_mutex_free (app_context->free_oids_lock);
   g_list_free (app_context->free_oid_blocks);
 
+  gzochid_application_event_source_free (app_context->event_source);
+  free (app_context->stats);
+
   free (context->fsm->name);
   free (context);
 }
 
+static void update_stats (gzochid_application_event *event, gpointer data)
+{
+  gzochid_stats_update_from_event ((gzochid_application_stats *) data, event);
+}
+
 void gzochid_application_context_init 
 (gzochid_application_context *context, gzochid_context *parent, 
  gzochid_application_descriptor *descriptor)
@@ -456,6 +639,8 @@ void gzochid_application_context_init
   gzochid_fsm_add_transition
     (fsm, GZOCHID_APPLICATION_STATE_PAUSED, GZOCHID_APPLICATION_STATE_STOPPED);
 
+  gzochid_fsm_on_enter 
+    (fsm, GZOCHID_APPLICATION_STATE_INITIALIZING, initialize_auth, context);
   gzochid_fsm_on_enter 
     (fsm, GZOCHID_APPLICATION_STATE_INITIALIZING, initialize_data, context);
   gzochid_fsm_on_enter 
@@ -470,6 +655,9 @@ void gzochid_application_context_init
   context->authenticator = gzochid_auth_function_pass_thru;
   context->descriptor = descriptor;
   
+  gzochid_application_event_attach
+    (context->event_source, update_stats, context->stats);
+
   gzochid_context_init ((gzochid_context *) context, parent, fsm);
 }
 
@@ -710,5 +898,5 @@ gzochid_auth_identity *gzochid_get_current_identity (void)
 void gzochid_register_client_received_message_task_serialization (void)
 {
   gzochid_task_register_serialization 
-    ("received-message", &gzochid_client_received_message_task_serialization);
+    (&gzochid_client_received_message_task_serialization);
 }
