@@ -24,7 +24,9 @@
 
 #include "auth.h"
 #include "context.h"
+#include "event.h"
 #include "io.h"
+#include "stats.h"
 #include "storage.h"
 #include "tx.h"
 
@@ -54,6 +56,9 @@ typedef struct _gzochid_application_descriptor
   gzochid_application_callback *initialized;
   gzochid_application_callback *logged_in;
 
+  char *auth_type;
+  GHashTable *auth_properties;
+
   GHashTable *properties;
 } gzochid_application_descriptor;
 
@@ -62,9 +67,11 @@ typedef struct _gzochid_application_context
   gzochid_context base;
 
   gzochid_application_descriptor *descriptor;
-  gzochid_auth_identity *(*authenticator)
-    (struct _gzochid_application_context *, unsigned char *, short);
 
+  gzochid_auth_identity *(*authenticator) 
+    (unsigned char *, short, gpointer, GError **);
+  gpointer auth_data;
+  
   GList *free_oid_blocks;
   GMutex *free_oids_lock;
 
@@ -75,6 +82,9 @@ typedef struct _gzochid_application_context
   GHashTable *oids_to_clients;
   GHashTable *clients_to_oids;
   GMutex *client_mapping_lock;
+  
+  gzochid_application_event_source *event_source;
+  gzochid_application_stats *stats;
 } gzochid_application_context;
 
 typedef void (*gzochid_application_worker) 
@@ -121,6 +131,12 @@ void gzochid_serialize_application_task
 (gzochid_application_context *, gzochid_application_task_serialization *, 
  gzochid_application_task *, GString *);
 
+gzochid_transaction_result gzochid_application_transaction_execute 
+(gzochid_application_context *, void (*) (gpointer), gpointer);
+
+gzochid_transaction_result gzochid_application_transaction_execute_timed 
+(gzochid_application_context *, void (*) (gpointer), gpointer, struct timeval);
+
 gzochid_transactional_application_task_execution *
 gzochid_transactional_application_task_execution_new 
 (gzochid_application_task *);
@@ -143,19 +159,6 @@ void gzochid_application_task_thread_worker (gpointer, gpointer);
 gboolean gzochid_application_should_retry 
 (gzochid_transactional_application_task_execution *);
 
-typedef struct _gzochid_durable_application_task
-{
-  gzochid_application_task *task;
-  gzochid_application_task_serialization *serialization;
-
-  gboolean repeats;
-  long period;
-
-  struct timeval target_execution_time;
-
-  mpz_t oid;
-} gzochid_durable_application_task;
-
 gzochid_io_serialization gzochid_application_callback_serialization;
 
 gzochid_application_callback *gzochid_application_callback_new 
