@@ -104,6 +104,9 @@ static gboolean flush_reference
 (gzochid_data_managed_reference *reference,
  gzochid_data_transaction_context *context)
 {
+  gzochid_application_data_event *event = NULL;
+  gzochid_application_event *base_event = NULL;
+
   char *oid_str = NULL;
   GString *out = NULL;
 
@@ -135,6 +138,16 @@ static gboolean flush_reference
 	(context->oids_transaction, oid_str, strlen (oid_str) + 1, 
 	 out->str, out->len);
 
+      event = malloc (sizeof (gzochid_application_data_event));
+      base_event = (gzochid_application_event *) event;
+
+      base_event->type = BYTES_WRITTEN;
+      gettimeofday (&base_event->timestamp, NULL);
+      event->bytes = out->len;
+
+      gzochid_application_event_dispatch
+	(context->context->event_source, base_event);
+
       g_string_free (out, FALSE);
 
       break;
@@ -446,7 +459,7 @@ void dereference
   char *oid_str = NULL; 
   char *data = NULL; 
   GString *in = NULL;
-
+  
   if (reference->obj != NULL 
       || reference->state == GZOCHID_MANAGED_REFERENCE_STATE_REMOVED_EMPTY)
     return;
@@ -467,6 +480,18 @@ void dereference
     }
   else 
     {
+      gzochid_application_data_event *event = 
+	malloc (sizeof (gzochid_application_data_event));
+      gzochid_application_event *base_event = 
+	(gzochid_application_event *) event;
+
+      base_event->type = BYTES_READ;
+      gettimeofday (&base_event->timestamp, NULL);
+      event->bytes = data_len;
+
+      gzochid_application_event_dispatch
+	(context->context->event_source, base_event);
+
       in = g_string_new_len (data, data_len);
       reference->obj = reference->serialization->deserializer 
 	(context->context, in);
@@ -633,11 +658,11 @@ void gzochid_data_remove_binding
   free (oid_str);
 }
 
-int gzochid_data_binding_exists
+gboolean gzochid_data_binding_exists
 (gzochid_application_context *context, char *name)
 {
   mpz_t oid;
-  int ret = TRUE;
+  gboolean ret = TRUE;
 
   gzochid_data_transaction_context *tx_context = NULL;
 
@@ -692,7 +717,8 @@ static void application_persist_data
       sub_reference_holder.reference = NULL;
       sub_reference_holder.serialization = reference_holder->serialization;
 
-      gzochid_transaction_execute (persist_data, &sub_reference_holder);
+      gzochid_application_transaction_execute 
+	(context, persist_data, &sub_reference_holder);
       reference_holder->reference = sub_reference_holder.reference;
     }
 }
@@ -731,7 +757,8 @@ gzochid_data_managed_reference *gzochid_data_create_reference_sync
       reference_holder.reference = gzochid_data_create_reference_to_oid 
 	(context, serialization, reference_holder.reference->oid);
     }
-  else gzochid_transaction_execute (persist_data, &reference_holder);
+  else gzochid_application_transaction_execute 
+	 (context, persist_data, &reference_holder);
   return reference_holder.reference;
 }
 
@@ -784,122 +811,3 @@ void gzochid_data_mark
       free (data);
     }
 }
-
-typedef struct _gzochid_persistence_task_data
-{
-  gpointer data;
-  gzochid_io_serialization *serialization;
-  gzochid_oid_holder *holder;
-} gzochid_persistence_task_data;
-
-typedef struct _gzochid_prefix_binding_persistence_task_data
-{
-  gzochid_persistence_task_data base;
-
-  char *prefix;
-} gzochid_prefix_binding_persistence_task_data;
-
-static void init_persistence_task 
-(gzochid_persistence_task_data *task, gzochid_io_serialization *serialization, 
- gpointer data, gzochid_oid_holder *holder)
-{
-  task->data = data;
-  task->serialization = serialization;
-  task->holder = holder;
-} 
-
-static gzochid_prefix_binding_persistence_task_data *
-gzochid_prefix_binding_persistence_task_data_new
-(gzochid_io_serialization *serialization, gpointer data,
- gzochid_oid_holder *holder, char *prefix)
-{
-  gzochid_prefix_binding_persistence_task_data *task = 
-    malloc (sizeof (gzochid_prefix_binding_persistence_task_data));
-
-  init_persistence_task 
-    ((gzochid_persistence_task_data *) task, serialization, data, holder);
-  task->prefix = prefix;
-
-  return task;
-}
-
-void gzochid_persistence_task_data_free (gzochid_persistence_task_data *task)
-{
-  free (task);
-}
-
-static void persistence_task_worker
-(gzochid_application_context *context, gzochid_auth_identity *identity, 
- gpointer data)
-{
-  gzochid_persistence_task_data *persistence_task = 
-    (gzochid_persistence_task_data *) data;
-
-  gzochid_data_managed_reference *reference = 
-    gzochid_data_create_reference 
-    (context, persistence_task->serialization, persistence_task->data);
-  
-  mpz_set (persistence_task->holder->oid, reference->oid);
-}
-
-static void prefix_binding_persistence_task_worker
-(gzochid_application_context *context, gzochid_auth_identity *identity, 
- gpointer data)
-{
-  gzochid_prefix_binding_persistence_task_data *
-    prefix_binding_persistence_task = 
-    (gzochid_prefix_binding_persistence_task_data *) data;
-  gzochid_persistence_task_data *persistence_task =
-    (gzochid_persistence_task_data *) prefix_binding_persistence_task;
-  GString *binding = g_string_new (prefix_binding_persistence_task->prefix);
-  char *oid_str = NULL;
-
-  persistence_task_worker (context, identity, data);
-  
-  oid_str = mpz_get_str (NULL, 16, persistence_task->holder->oid);
-  g_string_append (binding, oid_str);
-  free (oid_str);
-
-  gzochid_data_set_binding_to_oid 
-    (context, binding->str, persistence_task->holder->oid);
-  g_string_free (binding, FALSE);
-}
-
-gzochid_task *gzochid_data_prefix_binding_persistence_task_new
-(gzochid_application_context *context, gzochid_auth_identity *identity, 
- gzochid_io_serialization *serialization, gpointer data, 
- gzochid_oid_holder *holder, char *prefix)
-{
-  gzochid_prefix_binding_persistence_task_data *task_data =
-    gzochid_prefix_binding_persistence_task_data_new 
-    (serialization, data, holder, prefix);
-  gzochid_application_task *transactional_task = 
-    gzochid_application_task_new 
-    (context, identity, prefix_binding_persistence_task_worker, task_data);
-  gzochid_transactional_application_task_execution *execution = 
-    gzochid_transactional_application_task_execution_new (transactional_task);
-  gzochid_application_task *application_task = gzochid_application_task_new 
-    (context, identity, 
-     gzochid_application_resubmitting_transactional_task_worker, execution);
-
-  return gzochid_task_immediate_new 
-    (gzochid_application_task_thread_worker, application_task);
-}
-
-void gzochid_data_prefix_binding_persistence_task_free (gzochid_task *task)
-{
-  gzochid_application_task *application_task = 
-    (gzochid_application_task *) task->data;
-  gzochid_transactional_application_task_execution *execution = 
-    (gzochid_transactional_application_task_execution *) 
-    application_task->data;
-  gzochid_application_task *transactional_task = execution->task;
-  gzochid_persistence_task_data *data = 
-    (gzochid_persistence_task_data *) transactional_task->data;
-  
-  free (data);
-  free (transactional_task);
-  gzochid_transactional_application_task_execution_free (execution);
-  free (application_task);
-  free (task);
-}
