@@ -26,6 +26,7 @@
 #include <syslog.h>
 
 #include "app.h"
+#include "auth_int.h"
 #include "config.h"
 #include "context.h"
 #include "fsm.h"
@@ -40,12 +41,23 @@
 
 #define DEFAULT_TX_TIMEOUT_MS 100
 
+static void initialize_application 
+(gzochid_game_context *context, gzochid_application_descriptor *descriptor)
+{
+  gzochid_application_context *application_context =
+    gzochid_application_context_new ();
+  
+  gzochid_application_context_init
+    (application_context, (gzochid_context *) context, descriptor);
+  g_source_attach ((GSource *) application_context->event_source, 
+		   g_main_loop_get_context (context->event_loop));
+}
+
 static void scan_app_dir (gzochid_game_context *context, char *dir)
 {
   char *descriptor_file = g_strconcat (dir, "/", GAME_DESCRIPTOR_XML, NULL);
 
   gzochid_application_descriptor *descriptor = NULL;
-  gzochid_application_context *application_context = NULL;
 
   if (!g_file_test (descriptor_file, G_FILE_TEST_IS_REGULAR))
     {
@@ -54,11 +66,8 @@ static void scan_app_dir (gzochid_game_context *context, char *dir)
       return;
     }
 
-  application_context = gzochid_application_context_new ();
-  
   descriptor = gzochid_config_parse_application_descriptor (descriptor_file);
-  gzochid_application_context_init 
-    (application_context, (gzochid_context *) context, descriptor);
+  initialize_application (context, descriptor);
 }
 
 static void scan_apps_dir (gzochid_game_context *context)
@@ -83,6 +92,13 @@ static void initialize_async (gpointer data, gpointer user_data)
   gzochid_fsm_to_state (context->fsm, GZOCHID_GAME_STATE_RUNNING);
 }
 
+static void initialize_auth (int from_state, int to_state, gpointer user_data)
+{
+  gzochid_game_context *context = (gzochid_game_context *) user_data;
+
+  gzochid_auth_init (context);
+}
+
 static void initialize_server (int from_state, int to_state, gpointer user_data)
 {
   gzochid_game_context *context = (gzochid_game_context *) user_data;
@@ -143,6 +159,19 @@ static void initialize_complete
   gzochid_thread_pool_push (context->pool, initialize_async, NULL, NULL);  
 }
 
+static gpointer event_loop (gpointer data)
+{
+  gzochid_game_context *context = (gzochid_game_context *) data;
+  g_main_loop_run (context->event_loop);
+  return NULL;
+}
+
+static void initialize_event_loop 
+(int from_state, int to_state, gpointer user_data)
+{
+  g_thread_new ("event-loop", event_loop, user_data);
+}
+
 gzochid_game_context *gzochid_game_context_new (void)
 {
   return calloc (1, sizeof (gzochid_game_context));
@@ -151,39 +180,32 @@ gzochid_game_context *gzochid_game_context_new (void)
 void gzochid_game_context_free (gzochid_game_context *context)
 {
   gzochid_context_free ((gzochid_context *) context);
+
+  g_hash_table_destroy (context->applications);
+  g_hash_table_destroy (context->auth_plugins);
+
   free (context);
 } 
 
-gzochid_task_queue *gzochid_task_queue_new (GThreadPool *pool)
-{
-  gzochid_task_queue *task_queue = malloc (sizeof (gzochid_task_queue));
-  
-  task_queue->cond = g_cond_new ();
-  task_queue->mutex = g_mutex_new ();
-  task_queue->queue = g_queue_new ();
-
-  task_queue->pool = pool;
-  task_queue->consumer_thread = NULL;
-
-  return task_queue;
-}
-
 void gzochid_game_context_init
 (gzochid_game_context *context, gzochid_context *parent, GHashTable *config)
 {
   long tx_timeout_ms = 0;
   gzochid_fsm *fsm = gzochid_fsm_new 
     ("game", GZOCHID_GAME_STATE_INITIALIZING, "INITIALIZING");
+  GMainContext *event_context = g_main_context_new ();
 
   gzochid_fsm_add_state (fsm, GZOCHID_GAME_STATE_RUNNING, "RUNNING");
   gzochid_fsm_add_state (fsm, GZOCHID_GAME_STATE_STOPPED, "STOPPED");
 
+  context->event_loop = g_main_loop_new (event_context, FALSE);
+
   context->pool = gzochid_thread_pool_new 
     (context, 
      gzochid_config_to_int 
      (g_hash_table_lookup (config, "thread_pool.max_threads"), 4), 
      TRUE, NULL);
-  context->task_queue = gzochid_task_queue_new (context->pool);
+  context->task_queue = gzochid_schedule_task_queue_new (context->pool);
 
   gzochid_schedule_task_queue_start (context->task_queue);
 
@@ -202,6 +224,8 @@ void gzochid_game_context_init
   gzochid_fsm_add_transition
     (fsm, GZOCHID_GAME_STATE_RUNNING, GZOCHID_GAME_STATE_STOPPED);
 
+  gzochid_fsm_on_enter
+    (fsm, GZOCHID_GAME_STATE_INITIALIZING, initialize_event_loop, context);
   gzochid_fsm_on_enter 
     (fsm, GZOCHID_GAME_STATE_INITIALIZING, initialize_server, context);
   gzochid_fsm_on_enter 
@@ -212,12 +236,15 @@ void gzochid_game_context_init
   gzochid_fsm_on_enter 
     (fsm, GZOCHID_GAME_STATE_INITIALIZING, 
      initialize_client_received_message_task_serialization, context);
+  gzochid_fsm_on_enter 
+    (fsm, GZOCHID_GAME_STATE_INITIALIZING, initialize_auth, context);
   gzochid_fsm_on_enter 
     (fsm, GZOCHID_GAME_STATE_INITIALIZING, initialize_apps, context);
   gzochid_fsm_on_enter 
     (fsm, GZOCHID_GAME_STATE_INITIALIZING, initialize_complete, context);
   
   context->applications = g_hash_table_new (g_str_hash, g_str_equal);
+  context->auth_plugins = g_hash_table_new (g_str_hash, g_str_equal);
 
   gzochid_context_init ((gzochid_context *) context, parent, fsm);
 }
