@@ -1,5 +1,5 @@
 /* guile.c: GNU Guile interface routines for gzochid
- * Copyright (C) 2012 Julian Graham
+ * Copyright (C) 2013 Julian Graham
  *
  * gzochi is free software: you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by
@@ -22,6 +22,13 @@
 #include "guile.h"
 #include "threads.h"
 
+static SCM scm_r6rs_with_exception_handler;
+
+static SCM scm_r6rs_exception_handler;
+static SCM scm_exception_printer;
+static SCM scm_make_handler;
+static SCM scm_make_thunk;
+
 typedef struct _gzochid_guile_thread_work 
 {
   gzochid_thread_worker worker;
@@ -35,21 +42,15 @@ static SCM guile_catch_body (void *data)
 
   SCM procedure = (SCM) ptr[0];
   SCM args = (SCM) ptr[1];
+  SCM exception_var = (SCM) ptr[2];
   
-  return scm_apply_0 (procedure, args);
+  return scm_call_2 
+    (scm_r6rs_with_exception_handler,
+     scm_call_2 (scm_make_handler, scm_r6rs_exception_handler, exception_var),
+     scm_call_2 (scm_make_thunk, procedure, args));
 }
 
-static SCM guile_catch_handler (void *data, SCM tag, SCM throw_args)
-{
-  SCM exception_variable = (SCM) data;
-
-  if (scm_variable_p (exception_variable) == SCM_BOOL_T)
-    scm_variable_set_x (exception_variable, scm_cons (tag, throw_args));
-
-  return SCM_BOOL_F;
-}
-
-static SCM guile_pre_unwind_handler (void *data, SCM tag, SCM throw_args)
+static SCM r6rs_exception_handler (SCM exception_var, SCM cond)
 {
   scm_display_backtrace 
     (scm_make_stack (SCM_BOOL_T, SCM_EOL),
@@ -57,7 +58,41 @@ static SCM guile_pre_unwind_handler (void *data, SCM tag, SCM throw_args)
      SCM_BOOL_F, 
      SCM_BOOL_F);
 
-  scm_print_exception (scm_current_output_port (), SCM_BOOL_F, tag, throw_args);
+  scm_call_2 (scm_exception_printer, scm_current_output_port (), cond);
+
+  if (scm_variable_p (exception_var) == SCM_BOOL_T)
+    scm_variable_set_x (exception_var, cond);
+
+  return SCM_BOOL_F;
+}
+
+static SCM guile_catch_handler (void *data, SCM tag, SCM throw_args)
+{
+  SCM exception_variable = (SCM) data;
+
+  if (scm_variable_p (exception_variable) == SCM_BOOL_T
+      && scm_variable_ref (exception_variable) == SCM_UNSPECIFIED)
+    scm_variable_set_x (exception_variable, scm_cons (tag, throw_args));
+
+  return SCM_BOOL_F;
+}
+
+static SCM guile_pre_unwind_handler (void *data, SCM tag, SCM throw_args)
+{
+  SCM exception_variable = (SCM) data;
+
+  if (scm_variable_p (exception_variable) == SCM_BOOL_T
+      && scm_variable_ref (exception_variable) == SCM_UNSPECIFIED)
+    {
+      scm_display_backtrace 
+	(scm_make_stack (SCM_BOOL_T, SCM_EOL),
+	 scm_current_output_port (), 
+	 SCM_BOOL_F, 
+	 SCM_BOOL_F);
+  
+      scm_print_exception 
+	(scm_current_output_port (), SCM_BOOL_F, tag, throw_args);
+    }
 
   return SCM_BOOL_F;
 }
@@ -65,16 +100,17 @@ static SCM guile_pre_unwind_handler (void *data, SCM tag, SCM throw_args)
 SCM gzochid_guile_invoke (SCM procedure, SCM args, SCM exception_var)
 {
   SCM ret = SCM_BOOL_F;
-  void *sub_data[2];
+  void *sub_data[3];
 
   sub_data[0] = procedure;
   sub_data[1] = args;
+  sub_data[2] = exception_var;
 
   ret = scm_c_catch 
     (SCM_BOOL_T,
      guile_catch_body, sub_data, 
      guile_catch_handler, exception_var, 
-     guile_pre_unwind_handler, NULL);
+     guile_pre_unwind_handler, exception_var);
 
   scm_remember_upto_here_1 (procedure);
   scm_remember_upto_here_1 (args);
@@ -120,3 +156,30 @@ void gzochid_guile_thread_pool_push
 
   gzochid_thread_pool_push (pool, guile_dispatch, work, error);
 }
+
+static void bind_scm (char *module, SCM *binding, char *name)
+{
+  SCM var = scm_c_public_variable (module, name);
+
+  if (scm_is_false (var))
+    g_error ("Missing Scheme binding for `%s'. Aborting...", name);
+
+  *binding = scm_variable_ref (var);
+  scm_gc_protect_object (*binding);
+}
+
+void gzochid_guile_init ()
+{
+  scm_r6rs_exception_handler = scm_c_make_gsubr 
+    ("r6rs_exception_handler", 2, 0, 0, r6rs_exception_handler);
+  scm_gc_protect_object (scm_r6rs_exception_handler);
+
+  bind_scm ("rnrs exceptions", &scm_r6rs_with_exception_handler, 
+	    "with-exception-handler");
+
+  bind_scm ("gzochi private guile", &scm_exception_printer, 
+	    "gzochi:exception-printer");
+  bind_scm ("gzochi private guile", &scm_make_handler, "gzochi:make-handler");
+  bind_scm ("gzochi private guile", &scm_make_thunk, "gzochi:make-thunk");
+
+}
