@@ -20,16 +20,16 @@
 
 #include <glib.h>
 
+#include "admin.h"
 #include "context.h"
-#include "fsm.h"
 
 typedef struct _gzochid_server_context
 {
   gzochid_context base;
   GThreadPool *pool;
 
-  gzochid_admin_context *admin_context;
-  gzochid_game_context *game_context;
+  gzochid_context *admin_context;
+  gzochid_context *game_context;
 } gzochid_server_context;
 
 enum gzochid_state 
