@@ -1,5 +1,5 @@
 /* httpd.c: Embedded informational web server for gzochid
- * Copyright (C) 2011 Julian Graham
+ * Copyright (C) 2013 Julian Graham
  *
  * gzochi is free software: you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by
@@ -15,18 +15,18 @@
  * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
+#include <ctype.h>
 #include <glib.h>
 #include <gmp.h>
-
-#include <ctype.h>
+#include <stddef.h>
+#include <stdint.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
-
-#include <stdint.h>
 #include <sys/types.h>
 #include <sys/select.h>
 #include <sys/socket.h>
+
 #include <microhttpd.h>
 
 #include "admin.h"
@@ -37,6 +37,7 @@
 #include "gzochid.h"
 #include "log.h"
 #include "httpd.h"
+#include "stats.h"
 #include "storage.h"
 #include "threads.h"
 
@@ -44,6 +45,13 @@
 #define OID_SUFFIX_LEN 29
 #define OID_LINE_LEN 80
 
+#define GREETING \
+  "<h1>Hello, browser!</h1>\n" \
+  "<p>This is the administrative web console for the gzochid game " \
+  "application server. You can use this console to interrogate the state of " \
+  "applications running within container, including their attached data " \
+  "stores.</p>"
+
 struct data_state 
 {
   char *data;
@@ -63,7 +71,7 @@ static void next_line (struct data_state *state)
   gboolean needs_prefix = state->data_offset == 0;
   gboolean needs_suffix = remaining <= 16;
 
-  int line_length = OID_LINE_LEN - (16 - num_bytes);
+  int line_length = OID_LINE_LEN - (16 - num_bytes) + 1;
   
   if (state->data_offset == state->data_length)
     return;
@@ -74,7 +82,7 @@ static void next_line (struct data_state *state)
     line_length += OID_SUFFIX_LEN;
   
   state->current_line = malloc (sizeof (char) * (line_length + 1));
-  state->current_line_length = line_length + 1;
+  state->current_line_length = line_length;
 
   buf = state->current_line;
 
@@ -170,9 +178,29 @@ static void free_data_state (void *ptr)
   free (state);
 }
 
+static int not_found404 
+(struct MHD_Connection *connection, const char *page, 
+ int must_free, int must_copy)
+{
+  struct MHD_Response *response = MHD_create_response_from_data 
+    (strlen (page), (void *) page, must_free, must_copy);
+  int ret = MHD_queue_response (connection, 404, response);
+
+  MHD_destroy_response (response);
+
+  return ret;
+}
+
+static int not_found404_default (struct MHD_Connection *connection)
+{
+  return not_found404 
+    (connection, "<html><body>Not found.</body></html>", FALSE, FALSE);
+}
+
 static int dispatch_oid (struct MHD_Connection *connection, 
 			 gzochid_application_context *context, mpz_t oid)
 {
+  int ret = 0;
   size_t data_length = 0;
   char *oid_str = mpz_get_str (NULL, 16, oid);
   char *data = gzochid_storage_get 
@@ -187,39 +215,17 @@ static int dispatch_oid (struct MHD_Connection *connection,
     (-1, OID_PREFIX_LEN + OID_LINE_LEN + OID_SUFFIX_LEN, write_data_line, 
      state, free_data_state);
 
-  return MHD_queue_response (connection, MHD_HTTP_OK, response);
+  ret = MHD_queue_response (connection, MHD_HTTP_OK, response);
+  MHD_destroy_response (response);
+  return ret;
 }
 
 static int dispatch_oids 
-(struct MHD_Connection *connection, const char *url, const char *path,
+(struct MHD_Connection *connection, const char *url,
  gzochid_application_context *context)
 {
-  if (path == NULL)
-    {
-      GString *url_str = g_string_new (url);
-      struct MHD_Response *response = MHD_create_response_from_data 
-	(0, "", FALSE, FALSE);
-      
-      g_string_append (url_str, "/");
-      MHD_add_response_header (response, "Location", url_str->str);
-      g_string_free (url_str, TRUE);
-      
-      return MHD_queue_response 
-	(connection, MHD_HTTP_MOVED_PERMANENTLY, response);
-    }
-  else if (strlen (path) > 1)
-    {
-      mpz_t oid;
-      int ret = 0;
-
-      mpz_init (oid);
-      mpz_set_str (oid, path + 1, 16);
-      ret = dispatch_oid (connection, context, oid);
-      mpz_clear (oid);
-
-      return ret;
-    }
-  else
+  int ret = 0;
+  if (strlen (url) == 0)
     {
       GString *response_str = g_string_new (NULL);
       struct MHD_Response *response = NULL;
@@ -229,6 +235,8 @@ static int dispatch_oids
       
       g_string_append (response_str, "<html>\n");
       g_string_append (response_str, "  <body>\n");
+      g_string_append_printf 
+	(response_str, "    <h1>%s</h1><br />\n", context->descriptor->name);
       g_string_append (response_str, "    <ul>\n");
       
       while (k != NULL)
@@ -251,16 +259,36 @@ static int dispatch_oids
       g_string_append (response_str, "  </body>\n");
       g_string_append (response_str, "</html>");
       
-      response = MHD_create_response_from_data 
-	(response_str->len, response_str->str, TRUE, TRUE);
+      response = MHD_create_response_from_data
+	(response_str->len, response_str->str, TRUE, FALSE);
+      g_string_free (response_str, FALSE);
       
-      return MHD_queue_response (connection, MHD_HTTP_OK, response);
+      ret = MHD_queue_response (connection, MHD_HTTP_OK, response);
+      MHD_destroy_response (response);
+      return ret;
+    }
+  else
+    {
+      mpz_t oid;
+
+      mpz_init (oid);
+      if (mpz_set_str (oid, url, 16) < 0)
+	{	  
+	  mpz_clear (oid);
+	  return not_found404_default (connection);
+	}
+
+      ret = dispatch_oid (connection, context, oid);
+      mpz_clear (oid);
+
+      return ret;
     }
 }
 
 static int dispatch_names (struct MHD_Connection *connection, 
 			   gzochid_application_context *context)
 {
+  int ret = 0;
   GString *response_str = g_string_new (NULL);
   struct MHD_Response *response = NULL;
   
@@ -269,6 +297,8 @@ static int dispatch_names (struct MHD_Connection *connection,
 
   g_string_append (response_str, "<html>\n");
   g_string_append (response_str, "  <body>\n");
+  g_string_append_printf 
+    (response_str, "    <h1>%s</h1><br />\n", context->descriptor->name);
   g_string_append (response_str, "    <ul>\n");
 
   while (k != NULL)
@@ -290,39 +320,162 @@ static int dispatch_names (struct MHD_Connection *connection,
   g_string_append (response_str, "</html>");
 
   response = MHD_create_response_from_data 
-    (response_str->len, response_str->str, TRUE, TRUE);
+    (response_str->len, response_str->str, TRUE, FALSE);
+  g_string_free (response_str, FALSE);
+  
+  ret = MHD_queue_response (connection, MHD_HTTP_OK, response);
+  MHD_destroy_response (response);
+  return ret;
+}
 
-  return MHD_queue_response (connection, MHD_HTTP_OK, response);
+static int list_apps 
+(struct MHD_Connection *connection, gzochid_game_context *game_context)
+{
+  struct MHD_Response *response = NULL;
+  GList *apps = gzochid_game_context_get_applications (game_context);
+
+  if (apps == NULL)
+    return not_found404 
+      (connection, "<html><body>No applications.</body></html>", FALSE, FALSE);
+  else 
+    {
+      int ret = 0;
+      GList *apps_ptr = apps;
+      GString *response_str = g_string_new (NULL);
+
+      g_string_append (response_str, "<html>\n");
+      g_string_append (response_str, "  <body>\n");
+      g_string_append (response_str, "    <ul>\n");
+      
+      while (apps_ptr != NULL)
+	{
+	  gzochid_application_context *app = 
+	    (gzochid_application_context *) apps_ptr->data;
+	  
+  	  g_string_append_printf 
+	    (response_str, 
+	     "      <li><a href=\"/app/%s/\">%s</a></li><br />\n", 
+	     app->descriptor->name, app->descriptor->name);
+
+	  apps_ptr = apps_ptr->next;
+	}
+
+      g_string_append (response_str, "    </ul>\n");
+      g_string_append (response_str, "  </body>\n");
+      g_string_append (response_str, "</html>");
+
+      response = MHD_create_response_from_data
+	(response_str->len, response_str->str, TRUE, FALSE);
+      g_string_free (response_str, FALSE);
+
+      ret = MHD_queue_response (connection, MHD_HTTP_OK, response);
+      MHD_destroy_response (response);
+      return ret;
+    }
+}
+
+static int app_info 
+(struct MHD_Connection *connection, gzochid_application_context *app_context)
+{
+  int ret = 0;
+  struct MHD_Response *response = NULL;
+  GString *response_str = g_string_new (NULL);
+
+  g_string_append (response_str, "<html>\n");
+  g_string_append (response_str, "  <body>\n");
+
+  g_string_append_printf 
+    (response_str, "    <h1>%s</h1>\n", app_context->descriptor->name);
+  g_string_append_printf 
+    (response_str, "    <p>%s</p>\n", app_context->descriptor->description);
+  g_string_append (response_str, "    <h2>Application data</h2>\n");
+  g_string_append_printf 
+    (response_str, "    <a href=\"/app/%s/names/\">names</a><br />\n", 
+     app_context->descriptor->name);
+  g_string_append_printf 
+    (response_str, "    <a href=\"/app/%s/oids/\">oids</a><br />\n", 
+     app_context->descriptor->name);
+
+  g_string_append (response_str, "    <h2>Application statistics</h2>\n");
+  g_string_append (response_str, "    <table>\n");
+
+  g_string_append (response_str, "      <tr>\n");
+  g_string_append (response_str, "        <td>Bytes read</td>\n");
+  g_string_append_printf (response_str, "        <td>%lu</td>\n", 
+			  app_context->stats->bytes_read);
+  g_string_append (response_str, "      </tr>\n");
+  g_string_append (response_str, "      <tr>\n");
+  g_string_append (response_str, "        <td>Bytes written</td>\n");
+  g_string_append_printf (response_str, "        <td>%lu</td>\n", 
+			  app_context->stats->bytes_written);
+  g_string_append (response_str, "      </tr>\n");
+
+  g_string_append (response_str, "      <tr>\n");
+  g_string_append (response_str, "        <td>Transactions started</td>\n");
+  g_string_append_printf (response_str, "        <td>%u</td>\n", 
+			  app_context->stats->num_transactions_started);
+  g_string_append (response_str, "      </tr>\n");
+  g_string_append (response_str, "      <tr>\n");
+  g_string_append (response_str, "        <td>Transactions committed</td>\n");
+  g_string_append_printf (response_str, "        <td>%u</td>\n", 
+			  app_context->stats->num_transactions_committed);
+  g_string_append (response_str, "      </tr>\n");
+  g_string_append (response_str, "      <tr>\n");
+  g_string_append 
+    (response_str, "        <td>Transactions rolled back</td>\n");
+  g_string_append_printf (response_str, "        <td>%u</td>\n", 
+			  app_context->stats->num_transactions_rolled_back);
+  g_string_append (response_str, "      </tr>\n");
+
+  g_string_append (response_str, "    </table>\n");
+  g_string_append (response_str, "  </body>\n");
+  g_string_append (response_str, "</html>");
+
+  response = MHD_create_response_from_data 
+    (response_str->len, response_str->str, TRUE, FALSE);
+  g_string_free (response_str, FALSE);
+
+  ret = MHD_queue_response (connection, MHD_HTTP_OK, response);
+  MHD_destroy_response (response);
+  return ret;
 }
 
 static int dispatch_app (struct MHD_Connection *connection, const char *url,
 			 gzochid_game_context *game_context)
 {
   gzochid_application_context *app_context = NULL;
-  const char *app_url = url + 5;
-  int app_len = 0;
-
-  char *rest = strchr (app_url, '/');
+  char *rest = strchr (url, '/');
   char *name = NULL;
+  int name_len = 0;
   
-  app_len = rest - app_url;
-  name = calloc (app_len + 1, sizeof (char));  
-  name = strncpy (name, app_url, app_len);
+  if (rest == NULL)
+    return not_found404_default (connection);
 
+  name_len = rest - url;
+  name = strndup (url, name_len);  
   app_context = gzochid_game_context_lookup_application (game_context, name);
   free (name);
-  
+
   if (app_context != NULL)
     {
-      char *operation = strchr (rest, '/');
-      if (strncmp (operation, "/names", 6) == 0)
+      if (strcmp (rest, "/") == 0)
+	return app_info (connection, app_context);
+      if (strncmp (rest, "/names/", 7) == 0)
 	return dispatch_names (connection, app_context);
-      else if (strncmp (operation, "/oids", 5) == 0)
+      else if (strncmp (rest, "/oids/", 6) == 0)
 	return dispatch_oids 
-	  (connection, url, strchr (operation + 1, '/'), app_context);
-      else return 404;
+	  (connection, rest + 6, app_context);
+      else return not_found404_default (connection);
     }
-  else return 404;
+  else return not_found404_default (connection);
+}
+
+static int dispatch_apps (struct MHD_Connection *connection, const char *url,
+			  gzochid_game_context *game_context)
+{
+  if (strlen (url) == 0)
+    return list_apps (connection, game_context);
+  else return dispatch_app (connection, url, game_context);
 }
 
 static int dispatch (void *cls, struct MHD_Connection *connection, 
@@ -330,7 +483,7 @@ static int dispatch (void *cls, struct MHD_Connection *connection,
 		     const char *upload_data, size_t *upload_data_size, 
 		     void **con_cls)
 {
-  int ret = MHD_NO;
+  int ret = 0;
   struct MHD_Response *response = NULL;
   gzochid_httpd_context *context = (gzochid_httpd_context *) cls;
 
@@ -339,10 +492,19 @@ static int dispatch (void *cls, struct MHD_Connection *connection,
 
   if (strcmp (url, "/") == 0)
     {
-      const char *page = "<html><body>Hello, browser!</body></html>";
+      GString *response_str = g_string_new (NULL);
+
+      g_string_append (response_str, "<html><body>");
+      g_string_append (response_str, GREETING);
+      g_string_append (response_str, "<a href=\"app/\">app/</a>");
+      g_string_append (response_str, "</body></html>");
+      
       response = MHD_create_response_from_data 
-	(strlen (page), (void *) page, TRUE, TRUE);
+	(response_str->len, (void *) response_str->str, TRUE, FALSE);
+      g_string_free (response_str, FALSE);
+
       ret = MHD_queue_response (connection, MHD_HTTP_OK, response);
+      MHD_destroy_response (response);
     }
   else if (strncmp (url, "/app/", 5) == 0)
     {
@@ -353,17 +515,9 @@ static int dispatch (void *cls, struct MHD_Connection *connection,
       gzochid_game_context *game_context = (gzochid_game_context *)
 	server_context->game_context;
 
-      return dispatch_app (connection, url, game_context);
+      return dispatch_apps (connection, url + 5, game_context);
     }
-  else 
-    {
-      const char *page = "<html><body>Not found.</body></html>";
-      response = MHD_create_response_from_data 
-	(strlen (page), (void *) page, TRUE, TRUE);
-      ret = MHD_queue_response (connection, 404, response);
-    }
-
-  MHD_destroy_response (response);
+  else return not_found404_default (connection);
 
   return ret;
 }
