@@ -1,5 +1,5 @@
 ;; gzochi/data.scm: Public exports for gzochi data API
-;; Copyright (C) 2012 Julian Graham
+;; Copyright (C) 2013 Julian Graham
 ;;
 ;; gzochi is free software: you can redistribute it and/or modify it
 ;; under the terms of the GNU General Public License as published by
@@ -97,7 +97,6 @@
 
     (fields (immutable ca gzochi:managed-pair-car)
 	    (immutable cd gzochi:managed-pair-cdr))
-    (nongenerative gzochi:managed-pair)
     (sealed #t))
 
   (define gzochi:car gzochi:managed-pair-car)
@@ -150,8 +149,7 @@
     gzochi:managed-serializable?)
    (fields (mutable callback-with-value 
 		    (serialization serialization-with-value-serialization)))
-   (nongenerative gzochi:managed-serializable)
-   (protocol (lambda (n)
+   (protocol (lambda (p)
 	       (lambda (value serializer-callback deserializer-callback)
 		 (or (gzochi:callback? serializer-callback)
 		     (raise (condition 
@@ -162,10 +160,7 @@
 			     (make-assertion-violation)
 			     (make-irritants-condition deserializer-callback))))
 
-		 (let ((p (n)))
-		   (p (list serializer-callback 
-			    deserializer-callback 
-			    value)))))))
+		 (p (list serializer-callback deserializer-callback value))))))
  
   (define (gzochi:managed-serializable-serializer obj)
     (car (gzochi:managed-serializable-callback-with-value obj)))
@@ -202,10 +197,8 @@
 		    managed-vector-entry-wrapped-value-set!
 		    (serialization gzochi:boolean-serialization)))
 
-   (nongenerative gzochi:managed-vector-entry)
-
    (protocol 
-    (lambda (n)
+    (lambda (p)
       (lambda (value value-serializer value-deserializer)
 	(or (gzochi:managed-record? value)
 	    (and (gzochi:callback? value-serializer)
@@ -215,8 +208,7 @@
 		    (make-message-condition 
 		     "Serialization must be specified for unmanaged values."))))
 
-	(let* ((p (n))
-	       (wrapped-value (not (gzochi:managed-record? value)))
+	(let* ((wrapped-value (not (gzochi:managed-record? value)))
 	       (value (if wrapped-value
 			  (gzochi:make-managed-serializable
 			   value value-serializer value-deserializer)
@@ -262,8 +254,7 @@
 				      serialize-managed-vector
 				      deserialize-managed-vector))))
 
-   (nongenerative gzochi:managed-vector)
-   (protocol (lambda (n) (lambda (l) (let ((p (n))) (p (make-vector l #f))))))
+   (protocol (lambda (p) (lambda (l) (p (make-vector l #f)))))
    (sealed #t))
 
   (define (gzochi:managed-vector-length vec)
@@ -300,8 +291,7 @@
     (define constructor
       (gzochi:managed-record-constructor
        (gzochi:make-managed-record-constructor-descriptor
-	managed-vector #f 
-	(lambda (n) (lambda (vec) (let ((p (n))) (p vec)))))))
+	managed-vector #f (lambda (p) (lambda (vec) (p vec))))))
 
     (constructor 
      (list->vector 
@@ -333,9 +323,8 @@
 	    (serialization gzochi:boolean-serialization))
 	   (mutable next))
 
-   (nongenerative gzochi:managed-hashtable-entry)
    (protocol 
-    (lambda (n)
+    (lambda (p)
       (lambda (hash 
 	       key key-serializer key-deserializer 
 	       value value-serializer value-deserializer)
@@ -354,8 +343,7 @@
 		    (make-message-condition 
 		     "Serialization must be specified for unmanaged values."))))
 
-	(let* ((p (n))
-	       (wrapped-key (not (gzochi:managed-record? key)))
+	(let* ((wrapped-key (not (gzochi:managed-record? key)))
 	       (key (if wrapped-key
 			(gzochi:make-managed-serializable 
 			 key key-serializer key-deserializer)
@@ -412,11 +400,9 @@
 	   (immutable depth (serialization gzochi:integer-serialization))
 	   (mutable size (serialization gzochi:integer-serialization)))
 
-   (nongenerative gzochi:managed-hashtable-node)
-   (protocol (lambda (n)
+   (protocol (lambda (p)
 	       (lambda (depth)
-		 (let ((p (n)))
-		   (p #f #f #f #f (gzochi:make-managed-vector 256) depth 0)))))
+		 (p #f #f #f #f (gzochi:make-managed-vector 256) depth 0))))
    (sealed #t))
 
   (gzochi:define-managed-record-type 
@@ -426,13 +412,11 @@
 
     (fields hash-function equivalence-function root)
 
-    (nongenerative gzochi:managed-hashtable)
-    (protocol (lambda (n)
+    (protocol (lambda (p)
 		(lambda (hash-callback equiv-callback)
-		  (let ((p (n)))
-		    (let ((root (make-managed-hashtable-node 0)))
-		      (managed-hashtable-node-ensure-depth! root 6)
-		      (p hash-callback equiv-callback root))))))
+		  (let ((root (make-managed-hashtable-node 0)))
+		    (managed-hashtable-node-ensure-depth! root 6)
+		    (p hash-callback equiv-callback root)))))
     (sealed #t))
 
   (define (high-bits n num-bits)
