@@ -1,5 +1,5 @@
 /* storage.h: Prototypes and declarations for storage.c
- * Copyright (C) 2012 Julian Graham
+ * Copyright (C) 2013 Julian Graham
  *
  * gzochi is free software: you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by
@@ -21,18 +21,7 @@
 #include <gdbm.h>
 #include <glib.h>
 #include <gmp.h>
-
-enum gzochid_storage_operation_type
-  {
-    GZOCHID_STORAGE_OPERATION_PUT,
-    GZOCHID_STORAGE_OPERATION_DELETE
-  };
-
-enum gzochid_storage_lock_type
-  {
-    GZOCHID_STORAGE_LOCK_READ,
-    GZOCHID_STORAGE_LOCK_WRITE
-  };
+#include <sys/time.h>
 
 typedef struct _gzochid_storage_store
 {
@@ -45,6 +34,7 @@ typedef struct _gzochid_storage_transaction
   gzochid_storage_store *store;
   gpointer txn;
   gboolean rollback;
+  gboolean should_retry;
 } gzochid_storage_transaction;
 
 gzochid_storage_store *gzochid_storage_open (char *);
@@ -62,9 +52,11 @@ char *gzochid_storage_next_key
 
 gzochid_storage_transaction *gzochid_storage_transaction_begin
 (gzochid_storage_store *);
+gzochid_storage_transaction *gzochid_storage_transaction_begin_timed
+(gzochid_storage_store *, struct timeval);
 void gzochid_storage_transaction_commit (gzochid_storage_transaction *);
 void gzochid_storage_transaction_rollback (gzochid_storage_transaction *);
-void gzochid_storage_transaction_check (gzochid_storage_transaction *);
+void gzochid_storage_transaction_prepare (gzochid_storage_transaction *);
 char *gzochid_storage_transaction_get 
 (gzochid_storage_transaction *, char *, size_t, size_t *);
 char *gzochid_storage_transaction_get_for_update 
