@@ -18,11 +18,13 @@
 #include <assert.h>
 #include <glib.h>
 #include <libguile.h>
+#include <stddef.h>
 #include <stdlib.h>
 #include <string.h>
 
 #include "app.h"
 #include "auth.h"
+#include "auth_int.h"
 #include "data.h"
 #include "game.h"
 #include "guile.h"
@@ -37,6 +39,11 @@
 
 GHashTable *serialization_registry = NULL;
 
+typedef struct _gzochid_durable_application_task
+{
+  mpz_t handle_oid;
+} gzochid_durable_application_task;
+
 gzochid_application_task_serialization *
 gzochid_lookup_task_serialization (char *name)
 {
@@ -45,9 +52,10 @@ gzochid_lookup_task_serialization (char *name)
 }
 
 void gzochid_task_register_serialization 
-(char *name, gzochid_application_task_serialization *serialization)
+(gzochid_application_task_serialization *serialization)
 {
-  g_hash_table_insert (serialization_registry, name, serialization);
+  g_hash_table_insert 
+    (serialization_registry, serialization->name, serialization);
 }
 
 void gzochid_task_initialize_serialization_registry (void)
@@ -74,9 +82,6 @@ static gzochid_task_transaction_context *create_transaction_context
 static void cleanup_transaction (gzochid_task_transaction_context *tx_context)
 {
   g_list_free (tx_context->scheduled_tasks);
-  g_list_free_full (tx_context->scheduled_recurring_tasks, free);
-  g_list_free_full (tx_context->canceled_recurring_tasks, free);
-
   free (tx_context);
 }
 
@@ -114,63 +119,132 @@ gzochid_task *gzochid_task_make_transactional_application_task
 }
 
 gzochid_durable_application_task *gzochid_durable_application_task_new 
-(gzochid_application_task *task, 
- gzochid_application_task_serialization *serialization, 
- struct timeval target_execution_time)
+(mpz_t handle_oid)
 {
-  gzochid_durable_application_task *durable_task = 
+  gzochid_durable_application_task *task = 
     malloc (sizeof (gzochid_durable_application_task));
 
-  durable_task->task = task;
-  durable_task->serialization = serialization;
-  durable_task->target_execution_time = target_execution_time;
-  durable_task->repeats = FALSE;
+  mpz_init (task->handle_oid);
+  mpz_set (task->handle_oid, handle_oid);
 
-  mpz_init (durable_task->oid);
+  return task;
+}
+
+gzochid_durable_application_task_handle *create_durable_task_handle
+(gzochid_data_managed_reference *task_data_reference, 
+ gzochid_application_task_serialization *serialization,
+ gzochid_auth_identity *identity, struct timeval target_execution_time)
+{
+  gzochid_durable_application_task_handle *durable_task_handle = 
+    malloc (sizeof (gzochid_durable_application_task_handle));
+  struct timeval immediate = { 0, 0 };
+
+  durable_task_handle->task_data_reference = task_data_reference;
+  durable_task_handle->serialization = serialization;
+  durable_task_handle->identity = identity;
+  durable_task_handle->repeats = FALSE;
+  durable_task_handle->period = immediate;
+  durable_task_handle->target_execution_time = target_execution_time;
+
+  return durable_task_handle;
+}
+
+gzochid_durable_application_task_handle *create_durable_periodic_task_handle
+(gzochid_data_managed_reference *task_data_reference, 
+ gzochid_application_task_serialization *serialization, 
+ gzochid_auth_identity *identity, struct timeval target_execution_time, 
+ struct timeval period)
+{
+  gzochid_durable_application_task_handle *durable_task_handle = 
+    create_durable_task_handle 
+    (task_data_reference, serialization, identity, target_execution_time);
+
+  durable_task_handle->repeats = TRUE;
+  durable_task_handle->period = period;
   
-  return durable_task;
+  return durable_task_handle;
 }
 
 void gzochid_durable_application_task_free 
 (gzochid_durable_application_task *task)
 {
-  mpz_clear (task->oid);
   free (task);
 }
 
-gpointer deserialize_durable_task
+gpointer deserialize_durable_task_handle
 (gzochid_application_context *context, GString *in)
 {
-  char *serialization_name = gzochid_util_deserialize_string (in);
-  gzochid_application_task_serialization *serialization = 
-    gzochid_lookup_task_serialization (serialization_name);
-  gzochid_application_task *task =
-    gzochid_deserialize_application_task (context, serialization, in);
-  struct timeval target_execution_time = gzochid_util_deserialize_timeval (in);
+  gzochid_durable_application_task_handle *handle = 
+    malloc (sizeof (gzochid_durable_application_task_handle));
+  char *serialization_name = NULL;
 
-  return gzochid_durable_application_task_new 
-    (task, serialization, target_execution_time);
+  mpz_t oid;
+
+  mpz_init (oid);
+  gzochid_util_deserialize_mpz (in, oid);
+  
+  serialization_name = gzochid_util_deserialize_string (in);
+  handle->serialization = gzochid_lookup_task_serialization 
+    (serialization_name);
+
+  assert (handle->serialization != NULL);
+
+  handle->task_worker = 
+    handle->serialization->worker_serialization->deserializer (context, in);
+  handle->task_data_reference = gzochid_data_create_reference_to_oid 
+    (context, handle->serialization->data_serialization, oid);
+
+  mpz_clear (oid);
+ 
+  handle->identity = gzochid_auth_identity_deserializer (context, in);
+  handle->repeats = gzochid_util_deserialize_boolean (in);
+
+  if (handle->repeats)
+    handle->period = gzochid_util_deserialize_timeval (in);
+
+  handle->target_execution_time = gzochid_util_deserialize_timeval (in);
+  
+  free (serialization_name);
+  return handle;
 }
 
-void serialize_durable_task 
+void serialize_durable_task_handle
 (gzochid_application_context *context, gpointer data, GString *out)
 {
-  gzochid_durable_application_task *task = 
-    (gzochid_durable_application_task *) data;
+  gzochid_durable_application_task_handle *handle = 
+    (gzochid_durable_application_task_handle *) data;
 
-  gzochid_util_serialize_string (task->serialization->name, out);  
-  gzochid_serialize_application_task 
-    (context, task->serialization, task->task, out);
-  gzochid_util_serialize_timeval (task->target_execution_time, out);
+  gzochid_util_serialize_mpz (handle->task_data_reference->oid, out);
+  gzochid_util_serialize_string (handle->serialization->name, out);
+  handle->serialization->worker_serialization->serializer 
+    (context, handle->task_worker, out);
+
+  gzochid_auth_identity_serializer (context, handle->identity, out);
+
+  gzochid_util_serialize_boolean (handle->repeats, out);
+  if (handle->repeats)
+    gzochid_util_serialize_timeval (handle->period, out);
+
+  gzochid_util_serialize_timeval (handle->target_execution_time, out);
 }
 
-void finalize_durable_task 
+static void finalize_durable_task_handle
 (gzochid_application_context *context, gpointer data)
 {
+  gzochid_durable_application_task_handle *handle = 
+    (gzochid_durable_application_task_handle *) data;
+
+  gzochid_auth_identity_finalizer (context, handle->identity);
+  free (handle);
 }
 
-gzochid_io_serialization durable_task_serialization = 
-  { serialize_durable_task, deserialize_durable_task, finalize_durable_task };
+gzochid_io_serialization 
+gzochid_durable_application_task_handle_serialization = 
+  { 
+    serialize_durable_task_handle, 
+    deserialize_durable_task_handle, 
+    finalize_durable_task_handle
+  };
 
 gzochid_task *gzochid_task_new 
 (gzochid_thread_worker worker, gpointer data, 
@@ -197,37 +271,44 @@ gzochid_task *gzochid_task_immediate_new
 static void durable_task_application_worker
 (gzochid_application_context *, gzochid_auth_identity *, gpointer);
 
-static gzochid_task *rebuild_durable_task 
-(gzochid_application_context *context, mpz_t oid)
+gzochid_task *wrap_durable_task 
+(gzochid_application_context *context, gzochid_auth_identity *identity,
+ gzochid_durable_application_task *durable_task,
+ gzochid_durable_application_task_handle *durable_task_handle)
 {
   gzochid_game_context *game_context = (gzochid_game_context *) 
     ((gzochid_context *) context)->parent;
-
-  gzochid_data_managed_reference *task_reference = 
-    gzochid_data_create_reference_to_oid 
-    (context, &durable_task_serialization, oid);
-
-  gzochid_durable_application_task *task = NULL;
-  gzochid_application_task *transactional_task = NULL;
-  gzochid_transactional_application_task_execution *execution = NULL;
-  gzochid_application_task *application_task = NULL;
-
-  gzochid_data_dereference (task_reference);
-
-  task = (gzochid_durable_application_task *) task_reference->obj;
-  transactional_task = gzochid_application_task_new 
-    (context, task->task->identity, durable_task_application_worker, task);
-  execution = gzochid_transactional_application_task_timed_execution_new 
+  
+  gzochid_application_task *transactional_task = gzochid_application_task_new
+    (context, identity, durable_task_application_worker, durable_task);
+  gzochid_transactional_application_task_execution *execution = 
+    gzochid_transactional_application_task_timed_execution_new 
     (transactional_task, game_context->tx_timeout);
-  application_task = gzochid_application_task_new
-    (context, task->task->identity, 
+  gzochid_application_task *application_task = gzochid_application_task_new
+    (context, identity,
      gzochid_application_resubmitting_transactional_task_worker, execution);
 
-  mpz_set (task->oid, oid);
-
   return gzochid_task_new 
     (gzochid_application_task_thread_worker, application_task, 
-     task->target_execution_time);
+     durable_task_handle->target_execution_time);
+}
+
+static gzochid_task *rebuild_durable_task
+(gzochid_application_context *context, mpz_t oid)
+{
+  gzochid_durable_application_task *task = NULL;
+  gzochid_durable_application_task_handle *handle = NULL;
+  gzochid_data_managed_reference *handle_reference = 
+    gzochid_data_create_reference_to_oid 
+    (context, &gzochid_durable_application_task_handle_serialization, oid);
+
+  gzochid_data_dereference (handle_reference);
+
+  handle = (gzochid_durable_application_task_handle *) handle_reference->obj;
+  task = gzochid_durable_application_task_new (oid);
+
+  return wrap_durable_task
+    (context, handle->identity, task, handle);
 }
 
 static void commit_scheduled_task (gpointer data, gpointer user_data)
@@ -319,56 +400,53 @@ void gzochid_restart_tasks (gzochid_application_context *context)
   mpz_clear (oid);  
 }
 
-gzochid_task *build_durable_task 
+gzochid_durable_application_task_handle *build_durable_task_handle
 (gzochid_application_task *task, 
- gzochid_application_task_serialization *serialization,
- struct timeval delay)
+ gzochid_application_task_serialization *serialization, 
+ struct timeval delay, struct timeval *period)
 {
   struct timeval target;
 
+  gzochid_data_managed_reference *task_data_reference = NULL;
+  gzochid_durable_application_task_handle *durable_task_handle = NULL;
+  gzochid_data_managed_reference *handle_reference = NULL;
+
+  char *oid_str = NULL;
+  int oid_strlen, prefix_len;
+  char *binding_name;
+
   gettimeofday (&target, NULL);
 
   target.tv_sec += delay.tv_sec;
   target.tv_usec += delay.tv_usec;
 
-  {
-    gzochid_durable_application_task *durable_task = 
-      gzochid_durable_application_task_new (task, serialization, target);
-    gzochid_data_managed_reference *reference = gzochid_data_create_reference 
-      (task->context, &durable_task_serialization, durable_task);
+  task_data_reference = gzochid_data_create_reference 
+    (task->context, serialization->data_serialization, task->data);
+
+  if (period != NULL)
+    durable_task_handle = create_durable_periodic_task_handle
+      (task_data_reference, serialization, task->identity, target, *period);
+  else durable_task_handle = create_durable_task_handle
+	 (task_data_reference, serialization, task->identity, target);
+
+  handle_reference = gzochid_data_create_reference 
+    (task->context, &gzochid_durable_application_task_handle_serialization, 
+     durable_task_handle);
     
-    char *oid_str = mpz_get_str (NULL, 16, reference->oid);
-    int oid_str_len = strlen (oid_str), 
-      prefix_len = strlen (PENDING_TASK_PREFIX);
-    char *binding_name = calloc (oid_str_len + prefix_len + 1, sizeof (char));
- 
-    gzochid_game_context *game_context = (gzochid_game_context *) 
-      ((gzochid_context *) task->context)->parent;
-   
-    gzochid_application_task *transactional_task = gzochid_application_task_new
-      (task->context, task->identity, durable_task_application_worker, 
-       durable_task);
-    gzochid_transactional_application_task_execution *execution = 
-      gzochid_transactional_application_task_timed_execution_new 
-      (transactional_task, game_context->tx_timeout);
-    gzochid_application_task *application_task = gzochid_application_task_new
-      (task->context, task->identity, 
-       gzochid_application_resubmitting_transactional_task_worker, execution);
-    
-    gzochid_task *ret = gzochid_task_new 
-      (gzochid_application_task_thread_worker, application_task, target);
+  oid_str = mpz_get_str (NULL, 16, handle_reference->oid);
+  oid_strlen = strlen (oid_str);
+  prefix_len = strlen (PENDING_TASK_PREFIX);
+  binding_name = calloc (oid_strlen + prefix_len + 1, sizeof (char));
 
-    mpz_set (durable_task->oid, reference->oid);
-    binding_name = strncat (binding_name, PENDING_TASK_PREFIX, prefix_len);
-    binding_name = strncat (binding_name, oid_str, oid_str_len);
-    gzochid_data_set_binding_to_oid 
-      (task->context, binding_name, reference->oid);
-
-    free (oid_str);
-    free (binding_name);
-
-    return ret;
-  }
+  binding_name = strncat (binding_name, PENDING_TASK_PREFIX, prefix_len);
+  binding_name = strncat (binding_name, oid_str, oid_strlen);
+  gzochid_data_set_binding_to_oid 
+    (task->context, binding_name, handle_reference->oid);
+  
+  free (oid_str);
+  free (binding_name);
+  
+  return durable_task_handle;
 }
 
 static void durable_task_application_worker 
@@ -377,24 +455,58 @@ static void durable_task_application_worker
 {
   gzochid_durable_application_task *task = 
     (gzochid_durable_application_task *) data;
-  gzochid_application_task *inner_task = task->task;
-  
+
+  gzochid_data_managed_reference *handle_reference =
+    gzochid_data_create_reference_to_oid
+    (context, &gzochid_durable_application_task_handle_serialization, 
+     task->handle_oid);   
+  gzochid_durable_application_task_handle *handle = NULL;
+  gzochid_application_task *inner_task = NULL;
+
+  gzochid_data_dereference (handle_reference);
+
+  handle = (gzochid_durable_application_task_handle *) handle_reference->obj;
+  gzochid_data_dereference (handle->task_data_reference);
+  inner_task = gzochid_application_task_new 
+    (context, identity, handle->task_worker, handle->task_data_reference->obj);
+
   inner_task->worker (context, identity, inner_task->data);
 
-  { char *oid_str = mpz_get_str (NULL, 16, task->oid);
-    GString *binding = g_string_new (PENDING_TASK_PREFIX);
-    
-    g_string_append (binding, oid_str);
-    
-    gzochid_data_remove_binding (context, binding->str);
-    
-    g_string_free (binding, FALSE);
-    free (oid_str);
-  }
+  if (handle->repeats)
+    {
+      gzochid_task_transaction_context *tx_context = NULL;
+      gzochid_task *wrapped_task = NULL;
+      struct timeval now;
 
-  if (task->repeats) 
-    gzochid_schedule_durable_task 
-      (context, identity, inner_task, task->serialization);
+      gettimeofday (&now, NULL);
+      timeradd (&handle->period, 
+		&handle->target_execution_time, 
+		&handle->target_execution_time);
+
+      gzochid_data_mark 
+	(context, &gzochid_durable_application_task_handle_serialization, 
+	 handle);
+      wrapped_task = wrap_durable_task (context, identity, task, handle);
+
+      join_transaction (context);
+      tx_context = (gzochid_task_transaction_context *) 
+	gzochid_transaction_context (&task_participant);
+      
+      tx_context->scheduled_tasks = g_list_append 
+	(tx_context->scheduled_tasks, wrapped_task);
+    }
+  else
+    { 
+      char *oid_str = mpz_get_str (NULL, 16, task->handle_oid);
+      GString *binding = g_string_new (PENDING_TASK_PREFIX);
+      
+      g_string_append (binding, oid_str);
+      
+      gzochid_data_remove_binding (context, binding->str);
+      
+      g_string_free (binding, FALSE);
+      free (oid_str);
+    }
 }
 
 void gzochid_task_free (gzochid_task *task)
@@ -412,13 +524,9 @@ void gzochid_schedule_durable_task
     (context, identity, task, serialization, immediate);
 }
 
-void gzochid_schedule_delayed_durable_task
-(gzochid_application_context *context, gzochid_auth_identity *identity,
- gzochid_application_task *task, 
- gzochid_application_task_serialization *serialization, 
- struct timeval delay)
+static void schedule_durable_task 
+(gzochid_application_context *context, gzochid_task *task)
 {
-  gzochid_task *wrapped_task = build_durable_task (task, serialization, delay);
   gzochid_task_transaction_context *tx_context = NULL;
   
   join_transaction (context);
@@ -426,5 +534,51 @@ void gzochid_schedule_delayed_durable_task
     gzochid_transaction_context (&task_participant);
 
   tx_context->scheduled_tasks = g_list_append 
-    (tx_context->scheduled_tasks, wrapped_task);
+    (tx_context->scheduled_tasks, task);
+}
+
+void gzochid_schedule_delayed_durable_task
+(gzochid_application_context *context, gzochid_auth_identity *identity,
+ gzochid_application_task *task, 
+ gzochid_application_task_serialization *serialization, 
+ struct timeval delay)
+{
+  gzochid_durable_application_task_handle *handle = 
+    build_durable_task_handle (task, serialization, delay, NULL);
+  gzochid_data_managed_reference *handle_reference = 
+    gzochid_data_create_reference 
+    (context, &gzochid_durable_application_task_handle_serialization, handle);
+  gzochid_task *wrapped_task = wrap_durable_task
+    (context, identity,
+     gzochid_durable_application_task_new (handle_reference->oid), handle);
+
+  schedule_durable_task (context, wrapped_task);     
+}
+
+gzochid_periodic_task_handle *gzochid_schedule_periodic_durable_task 
+(gzochid_application_context *context, gzochid_auth_identity *identity, 
+ gzochid_application_task *task,
+ gzochid_application_task_serialization *serialization, 
+ struct timeval delay, struct timeval period)
+{
+  gzochid_durable_application_task_handle *handle = 
+    build_durable_task_handle (task, serialization, delay, &period);
+  gzochid_data_managed_reference *handle_reference = 
+    gzochid_data_create_reference 
+    (context, &gzochid_durable_application_task_handle_serialization, handle);
+  gzochid_task *wrapped_task = wrap_durable_task
+    (context, identity,
+     gzochid_durable_application_task_new (handle_reference->oid), handle);
+
+  schedule_durable_task (context, wrapped_task);
+
+  return handle;
+}
+
+void gzochid_cancel_periodic_task 
+(gzochid_application_context *context, gzochid_periodic_task_handle *handle)
+{
+  handle->repeats = FALSE;
+  gzochid_data_mark 
+    (context, &gzochid_durable_application_task_handle_serialization, handle);
 }
