@@ -24,14 +24,28 @@
 
 #include "app.h"
 #include "auth.h"
+#include "data.h"
 #include "io.h"
 #include "threads.h"
 
-typedef struct _gzochid_periodic_task_handle
+typedef struct _gzochid_durable_application_task_handle
 {
-  mpz_t descriptor_oid;
-  mpz_t scm_oid;
-} gzochid_periodic_task_handle;
+  gzochid_application_task_serialization *serialization;
+
+  gzochid_application_worker task_worker;
+  gzochid_data_managed_reference *task_data_reference;
+
+  gzochid_auth_identity *identity;
+  
+  gboolean repeats;
+  struct timeval period;
+  struct timeval target_execution_time;
+} gzochid_durable_application_task_handle;
+
+typedef gzochid_durable_application_task_handle gzochid_periodic_task_handle;
+
+gzochid_io_serialization 
+gzochid_durable_application_task_handle_serialization;
 
 typedef struct _gzochid_task
 {
@@ -47,8 +61,6 @@ typedef struct _gzochid_task_transaction_context
   gzochid_auth_identity *identity;
 
   GList *scheduled_tasks;
-  GList *scheduled_recurring_tasks;
-  GList *canceled_recurring_tasks;
 } gzochid_task_transaction_context;
 
 gzochid_task *gzochid_task_new 
@@ -77,11 +89,12 @@ gzochid_periodic_task_handle *gzochid_schedule_periodic_durable_task
  gzochid_application_task *, gzochid_application_task_serialization *, 
  struct timeval, struct timeval);
 
-void gzochid_cancel_periodic_task (gzochid_periodic_task_handle *);
+void gzochid_cancel_periodic_task 
+(gzochid_application_context *, gzochid_periodic_task_handle *);
 
 void gzochid_task_initialize_serialization_registry (void);
 void gzochid_task_register_serialization 
-(char *, gzochid_application_task_serialization *);
+(gzochid_application_task_serialization *);
 void gzochid_restart_tasks (gzochid_application_context *);
 
 #endif /* GZOCHID_TASK_H */
