@@ -1,5 +1,5 @@
 /* tx.c: Application-level transactions implementation for gzochid
- * Copyright (C) 2012 Julian Graham
+ * Copyright (C) 2013 Julian Graham
  *
  * gzochi is free software: you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by
@@ -18,13 +18,17 @@
 #include <assert.h>
 #include <glib.h>
 #include <gmp.h>
+#include <stddef.h>
 #include <stdlib.h>
+#include <sys/time.h>
 
 #include "log.h"
 #include "tx.h"
+#include "util.h"
 
 static GStaticMutex transaction_mutex = G_STATIC_MUTEX_INIT;
 static GStaticPrivate thread_transaction_key = G_STATIC_PRIVATE_INIT;
+static GStaticPrivate thread_transaction_timing_key = G_STATIC_PRIVATE_INIT;
 static mpz_t transaction_counter;
 static gboolean transactions_initialized;
 
@@ -38,12 +42,20 @@ enum gzochid_transaction_state
     GZOCHID_TRANSACTION_STATE_ROLLED_BACK
   };
 
+typedef struct _gzochid_transaction_timing
+{
+  struct timeval start_time;
+  struct timeval *timeout;
+} gzochid_transaction_timing;
+
 typedef struct _gzochid_transaction 
 {
   mpz_t id;
   char *name;
   GHashTable *participants;
   enum gzochid_transaction_state state;
+
+  gzochid_transaction_timing *timing;
 } gzochid_transaction;
 
 static gzochid_transaction *transaction_new (void)
@@ -82,6 +94,7 @@ typedef struct _gzochid_transaction_participant_registration
 
   gpointer data;
   gboolean rollback;
+  gboolean retryable;
 } gzochid_transaction_participant_registration;
 
 gzochid_transaction_participant *gzochid_transaction_participant_new 
@@ -111,7 +124,9 @@ void gzochid_transaction_join
     g_static_private_get (&thread_transaction_key);
   if (transaction == NULL)
     {
-      transaction = transaction_new ();   
+      transaction = transaction_new ();
+      transaction->timing = g_static_private_get 
+	(&thread_transaction_timing_key);
       g_static_private_set (&thread_transaction_key, transaction, NULL);
     }
   else assert (transaction->state == GZOCHID_TRANSACTION_STATE_ACTIVE);
@@ -214,6 +229,23 @@ static gboolean rollback_only (gzochid_transaction *transaction)
   return FALSE;
 }
 
+static gboolean retryable (gzochid_transaction *transaction)
+{
+  GHashTableIter iter;
+  gpointer key, value;
+  
+  g_hash_table_iter_init (&iter, transaction->participants);
+  while (g_hash_table_iter_next (&iter, &key, &value))
+    {
+      gzochid_transaction_participant_registration *participant = 
+	(gzochid_transaction_participant_registration *) value;
+      if (participant->rollback && !participant->retryable)
+	return FALSE;
+    }
+
+  return TRUE;
+}
+
 static void rollback (gzochid_transaction *transaction)
 {
   GList *participants = g_hash_table_get_values (transaction->participants);
@@ -236,7 +268,7 @@ static void rollback (gzochid_transaction *transaction)
 }
 
 void gzochid_transaction_mark_for_rollback 
-(gzochid_transaction_participant *participant)
+(gzochid_transaction_participant *participant, gboolean retryable)
 {
   gzochid_transaction *transaction = (gzochid_transaction *) 
     g_static_private_get (&thread_transaction_key);  
@@ -247,29 +279,119 @@ void gzochid_transaction_mark_for_rollback
 
   assert (registration != NULL);
   registration->rollback = TRUE;
+  registration->retryable = retryable;
 }
 
-int gzochid_transaction_execute (void (*func) (gpointer), gpointer data)
+static struct timeval time_remaining (gzochid_transaction_timing *timing)
+{
+  struct timeval now, ret, zero = { 0, 0 };
+
+  if (timing->timeout == NULL)
+    return zero;
+
+  gettimeofday (&now, NULL);
+  timersub (&now, &timing->start_time, &ret);
+  if (timercmp (&ret, timing->timeout, >))
+    ret = zero;
+  else 
+    {
+      struct timeval ret2;
+      timersub (timing->timeout, &ret, &ret2);
+      ret = ret2;
+    }
+
+  return ret; 
+}
+
+struct timeval gzochid_transaction_time_remaining ()
+{
+  gzochid_transaction_timing *timing = g_static_private_get 
+    (&thread_transaction_timing_key);
+
+  assert (timing != NULL);
+
+  return time_remaining (timing);  
+}
+
+static gzochid_transaction_result transaction_execute
+(void (*func) (gpointer), gpointer data, struct timeval *timeout)
 {
   gzochid_transaction *transaction = NULL;
+  gzochid_transaction_timing timing;
   gboolean success = TRUE;
-  
+  gboolean should_retry = FALSE;
+  int ms = 0;
+
+  /* Set up the transaction timing information here, even before any
+     participants have joined - the idea being that if a transaction does end
+     up getting created because a participant requiring transactional semantics
+     joins, then this we should consider it as having begun now. */
+
+  gettimeofday (&timing.start_time, NULL);
+  timing.timeout = timeout;
+
+  g_static_private_set (&thread_transaction_timing_key, &timing, NULL);
+
   func (data);
 
+  g_static_private_set (&thread_transaction_timing_key, NULL, NULL);
+
   transaction = g_static_private_get (&thread_transaction_key);
   if (transaction == NULL)
-    return TRUE;
+    return GZOCHID_TRANSACTION_SUCCESS;
 
   if (rollback_only (transaction) || !prepare (transaction))
     {
       rollback (transaction);
       success = FALSE;
     }
-  else commit (transaction);
-  g_static_private_set (&thread_transaction_key, NULL, NULL);
+  else 
+    {
+      struct timeval tx_finish;
+      struct timeval remaining = time_remaining (&timing);
 
+      if (timing.timeout != NULL 
+	  && remaining.tv_sec <= 0 
+	  && remaining.tv_usec <= 0)
+	{
+	  rollback (transaction);
+	  success = FALSE;
+	  should_retry = TRUE;
+	}
+      else commit (transaction);
+	  
+      gettimeofday (&tx_finish, NULL);
+      ms = ((tx_finish.tv_sec - timing.start_time.tv_sec) * 1000)
+	+ ((tx_finish.tv_usec - timing.start_time.tv_usec) / 1000);
+      
+      if (success)
+	gzochid_debug ("Transaction completed in %dms", ms);
+      else gzochid_warning ("Transaction timed out after %dms", ms);
+    }
+
+  g_static_private_set (&thread_transaction_key, NULL, NULL);
+  
+  if (!success)
+    should_retry |= retryable (transaction);
   transaction_free (transaction);
-  return success;
+  
+  if (success)
+    return GZOCHID_TRANSACTION_SUCCESS;
+  else return should_retry 
+	 ? GZOCHID_TRANSACTION_SHOULD_RETRY
+	 : GZOCHID_TRANSACTION_FAILURE;    
+}
+
+gzochid_transaction_result gzochid_transaction_execute 
+(void (*func) (gpointer), gpointer data)
+{
+  return transaction_execute (func, data, NULL);
+}
+
+gzochid_transaction_result gzochid_transaction_execute_timed
+(void (*func) (gpointer), gpointer data, struct timeval timeout)
+{
+  return transaction_execute (func, data, &timeout);
 }
 
 gboolean gzochid_transaction_active ()
@@ -289,3 +411,28 @@ gboolean gzochid_transaction_rollback_only ()
 
   return rollback_only (transaction);
 }
+
+gboolean gzochid_transaction_timed_out ()
+{
+  gzochid_transaction *transaction = (gzochid_transaction *) 
+    g_static_private_get (&thread_transaction_key);
+
+  assert (transaction != NULL);
+
+  if (transaction->timing->timeout != NULL)
+    {
+      struct timeval remaining = gzochid_transaction_time_remaining ();
+      return remaining.tv_sec <= 0 && remaining.tv_usec <= 0;
+    }
+  else return FALSE;
+}
+
+gboolean gzochid_transaction_timed ()
+{
+  gzochid_transaction_timing *timing = g_static_private_get 
+    (&thread_transaction_timing_key);
+
+  assert (timing != NULL);
+  return timing->timeout != NULL;
+}
+
